<#ibiztemplate>
TARGET=PSSYSTEM
</#ibiztemplate>
<#assign hasMQProducer=false>
<#list sys.getAllPSDataEntities() as dataEntity>
<#if dataEntity.getAllPSDEDataSyncs?? && dataEntity.getAllPSDEDataSyncs()??>
    <#list dataEntity.getAllPSDEDataSyncs() as dataSync>
        <#if dataSync.getOutPSSysDataSyncAgent?? && dataSync.getOutPSSysDataSyncAgent()??>
            <#assign hasMQProducer=true>
            <#break>
        </#if>
    </#list>
</#if>
</#list>
<#if hasMQProducer>
    <#comment>服务接口微服务平台配置</#comment>
    <#assign mqServerAddress="">
    <#if sys.getAllPSDevSlnMSDepAPIs()??>
        <#list sys.getAllPSDevSlnMSDepAPIs() as depSysApi>
            <#if depSysApi.getPSDCMSPlatform()?? >
                <#if depSysApi.getUserParam("rocketmq.producer.namesrvAddr","")??>
                    <#assign mqServerAddress = depSysApi.getUserParam("rocketmq.producer.namesrvAddr","")>
                </#if>
            </#if>
        </#list>
    </#if>
    <#if mqServerAddress!=''>
        <#assign mqServer="on">
    <#else>
        <#assign mqServer="off">
    </#if>
package ${pub.getPKGCodeName()}.util.aspect;

import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import ${pub.getPKGCodeName()}.util.domain.EntityBase;
import org.apache.rocketmq.client.producer.DefaultMQProducer;
import org.apache.rocketmq.client.producer.SendResult;
import org.apache.rocketmq.common.message.Message;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Lazy;
import org.springframework.expression.EvaluationContext;
import org.springframework.expression.Expression;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

/**
 * rocketMQ消息切面
 */
@Slf4j
@Aspect
@Component
@ConditionalOnExpression("'${r'${rocketmq.producer.isOnOff:'+mqServer+"}'.equals('on')"}")
public class RocketMQAspect
{
    private final ExpressionParser parser = new SpelExpressionParser();

    @Autowired
    @Lazy
    DefaultMQProducer defaultMQProducer;

    @Value("${r'${rocketmq.producer.topic:default}'}")
    private String topic;

    <#list sys.getAllPSDataEntities() as dataEntity>
        <#if dataEntity.getAllPSDEDataSyncs?? && dataEntity.getAllPSDEDataSyncs()??>
            <#list dataEntity.getAllPSDEDataSyncs() as dataSync>
                <#if dataSync.getOutPSSysDataSyncAgent?? && dataSync.getOutPSSysDataSyncAgent()??>
                    <#assign producer=dataEntity.codeName+dataSync.codeName>
    @Around(value = "${getPointCut(dataEntity,dataSync)}")
    public Object ${srfmethodname(producer)}(ProceedingJoinPoint point) throws Throwable{
        Object entity = getEntity(point);
        Object result = point.proceed();
        <#if dataSync.getOutTestPSDEAction?? && dataSync.getOutTestPSDEAction()??>
        <#assign actionName=srfmethodname(dataSync.getOutTestPSDEAction().codeName)>
        outputAction(point, "${actionName}", entity);
        </#if>
        sendMsg(topic, "${dataSync.codeName?lower_case}", entity);
        return result;
    }
                </#if>
            </#list>
        </#if>
    </#list>

    /**
     * 输出过滤行为
     * @param point
     * @param actionName
     * @param entity
     */
    private void outputAction(JoinPoint point, String actionName, Object entity) {
        Object[] args = point.getArgs();
        if (ObjectUtils.isEmpty(args) || args.length == 0 || ObjectUtils.isEmpty(entity)) {
            return;
        }
        Object service = point.getTarget();
        EvaluationContext serviceCtx = new StandardEvaluationContext();
        serviceCtx.setVariable("service", service);
        serviceCtx.setVariable("arg", entity);
        Expression serviceExp = parser.parseExpression(String.format("#service.%s(#arg)", actionName));
        serviceExp.getValue(serviceCtx);
    }

    /**
     * 获取实体对象
     *
     * @param point
     * @return
     */
    private Object getEntity(ProceedingJoinPoint point) {
        Object entity = null;
        try {
            String action = point.getSignature().getName();
            Object[] args = point.getArgs();
            Object serviceObj = point.getTarget();
            if (ObjectUtils.isEmpty(args) || args.length == 0 || StringUtils.isEmpty(action)) {
                return entity;
            }
            if ("remove".equalsIgnoreCase(action)) {
                Object idValue = args[0];
                if (!ObjectUtils.isEmpty(serviceObj)) {
                    EvaluationContext oldContext = new StandardEvaluationContext();
                    oldContext.setVariable("service", serviceObj);
                    oldContext.setVariable("id", idValue);
                    Expression oldExp = parser.parseExpression("#service.get(#id)");
                    entity = oldExp.getValue(oldContext, EntityBase.class);
                }
            } else {
                entity = args[0];
            }
        } catch (Exception e) {
            log.error("发送消息失败，无法获取实体对象" + e);
        }
        return entity;
    }

    /**
     * 发送消息
     *
     * @param topic
     * @param tag
     * @param body
     */
    private void sendMsg(String topic, String tag, Object body) {
        if (ObjectUtils.isEmpty(body)) {
            log.error("消息内容为空，[{}]消息将被忽略！", tag);
            return;
        }
        try {
            Message message = new Message(topic, tag, JSON.toJSONString(body).getBytes());
            SendResult sendResult = defaultMQProducer.send(message);
            log.info("消息发送响应：" + sendResult.toString());
        } catch (Exception e) {
            log.error("消息发送异常，" + e);
        }
    }
}
</#if>

<#comment>输出切点</#comment>
<#function getPointCut dataEntity dataSync>
    <#comment>数值代码表 1=新建; 2=编辑; 4=删除</#comment>
    <#assign createArray = [1,3,5,7]>
    <#assign updateArray = [2,3,6,7]>
    <#assign deleteArray=[4,5,6,7]>
    <#assign result="">
    <#if dataSync.getEventType?? && dataSync.getEventType()??>
        <#assign type=dataSync.getEventType()>
        <#assign isCreate=createArray?seq_contains(type)>
        <#assign isUpdate=updateArray?seq_contains(type)>
        <#assign isDelete=deleteArray?seq_contains(type)>
        <#if isCreate>
            <#assign result="execution(* "+pub.getPKGCodeName()+".core.*.service.I"+dataEntity.codeName+"Service.create*(..))">
        </#if>
        <#if isUpdate>
            <#if result!="">
                <#assign result=result+"||">
            </#if>
            <#assign result=result+"execution(* "+pub.getPKGCodeName()+".core.*.service.I"+dataEntity.codeName+"Service.update*(..))">
        </#if>
        <#if isDelete>
            <#if result!="">
                <#assign result=result+"||">
            </#if>
            <#assign result=result+"execution(* "+pub.getPKGCodeName()+".core.*.service.I"+dataEntity.codeName+"Service.remove*(..))">
        </#if>
    </#if>
    <#return "("+result+")">
</#function>